//
// UART1_WiFi_ESP8266_TCPclient : ESP8266 run as TCP client
//
// EVB : NuMaker Uni
// MCU : NANO100NE3BN
// WiFi: ESP-03 (ESP8266)
//
// UART1 to WiFi connections
// PB4/UART1-RX to WiFi's TX 
// PB5/UART1-TX to WiFi's RX

#include <stdio.h>
#include <string.h>
#include "Nano100Series.h"
#include "MCU_init.h"
#include "SYS_init.h"
#include "lx_step_counter.h"
#include "lx_data_manager.h"
#include "lx_Heart_rate.h"
#include "I2C_SSD1306Z.h"
#include "lx_lcd_show.h"
#include "HTU21D.h"	
#include "GY_282.h"
#include "lx_blu_communication.h"
#include "lx_wifi.h"
#include "lx_wall_clock_timer.h";

volatile char Text[16];
volatile uint8_t B1_Flag = 0;
volatile uint8_t B2_Flag = 0;
volatile uint8_t Show_Flag = 0;
int Blu_Is_Ok=0;
int i;

void TMR1_IRQHandler(void)
{
	for(i=0;i<10;i++)
	{
	   sensor_timer_handle();
     senser_heart_rate_handle();
	}
	TIMER_ClearIntFlag(TIMER1);  // clear timer interrupt flag
}

void Init_Timer1(void)
{    
    TIMER_Open(TIMER1, TMR1_OPERATING_MODE, TMR1_OPERATING_FREQ);
    TIMER_EnableInt(TIMER1);
    NVIC_EnableIRQ(TMR1_IRQn);
    TIMER_Start(TIMER1);
}
void Init_ADC(void)
{
    ADC_Open(ADC, ADC_INPUT_MODE, ADC_OPERATION_MODE, ADC_CHANNEL_MASK );
    ADC_POWER_ON(ADC);
    NVIC_EnableIRQ(ADC_IRQn);
    ADC_START_CONV(ADC);
}
void TMR0_IRQHandler(void)
{
	 HTU21D_handle();
	 if(Show_Flag==1)
	 {
		 GY282_Data_handle();
     Lcd_show_GY();
	 }else
		 Lcd_Show(); 
	 
//   Send_Data_To_Phone();		 
   TIMER_ClearIntFlag(TIMER0);  // clear timer interrupt flag
}

void Init_Timer0(void)
{
    TIMER_Open(TIMER0, TIMER_PERIODIC_MODE, TMR0_OPERATING_FREQ);
    TIMER_EnableInt(TIMER0);
    NVIC_EnableIRQ(TMR0_IRQn);
    TIMER_Start(TIMER0);
}
void GPABC_IRQHandler(void)
{
    if (PA->ISRC & BIT15) { // check if PA15 interrupt occurred
        PA->ISRC |= BIT15;  // clear PA15 interrupt status
			  B1_Flag=1;   			// set a flag for Button1
			  printf("%d",B1_Flag);
    }; 
		if (PA->ISRC & BIT14) { // check if PA14 interrupt occurred
        PA->ISRC |= BIT14;  // clear PA14 interrupt status  
        UART_Write(UART0,"danger",strlen("danger"));
			  CLK_SysTickDelay(20000);
			  printf("Show_Flag %d",Show_Flag);
    };
		if (PA->ISRC & BIT13) { // check if PA13 interrupt occurred
        PA->ISRC |= BIT13; 			// clear PA13 interrupt status 			
			  Show_Flag=!Show_Flag;
			  if(Show_Flag==1)
				{
			    TIMER_Stop(TIMER0);
			    TIMER_Stop(TIMER1);
          clear_LCD();
			    TIMER_Start(TIMER0);
				}
				if(Show_Flag==0)
				{
					clear_LCD();
					TIMER_Start(TIMER1);
				}
			  printf("Show_Flag %d",Show_Flag);
    };		
}
void Init_Buttons(void)
{
    GPIO_SetMode(PA, (BIT13 | BIT14 | BIT15), GPIO_PMD_INPUT);
    GPIO_ENABLE_PULL_UP(PA, (BIT13 | BIT14 | BIT15));	
    GPIO_EnableInt(PA, 15, GPIO_INT_FALLING);
    GPIO_EnableInt(PA, 14, GPIO_INT_FALLING);
  	GPIO_EnableInt(PA, 13, GPIO_INT_FALLING);			
    NVIC_EnableIRQ(GPABC_IRQn);
    GPIO_SET_DEBOUNCE_TIME(GPIO_DBCLKSRC_LIRC, GPIO_DBCLKSEL_64); // debounce clock source = HCLK or LIRC
    GPIO_ENABLE_DEBOUNCE(PA, (BIT13 | BIT14 | BIT15));	
}


int32_t main()
{
		SYS_Init();
	  Init_ADC();
	  Init_UART0();
	  Init_UART1();
	  I2C_Open(I2C0, I2C0_CLOCK_FREQUENCY);
	  I2C_Open(I2C1, I2C1_CLOCK_FREQUENCY);
	  Init_Buttons();
	  HTU21D_Init();
	  GY282_Init();
	  myuser.bit_field.age=20;
	  myuser.bit_field.hight=175;
	  myuser.bit_field.weight=77;
	  myuser.bit_field.gender=1;
	  start_health_algorithm(&myuser);
    init_LCD();
    clear_LCD();
	  Lcd_Show(); 
	  Init_RTC();
	  Init_Timer0();
	  Init_Timer1();   
    while(1) {	
//			Receive_Data_Handle();
//			if(Blu_Is_Ok==0)
//			{
//				BLE_Test();
//			}
//			Send_Data_To_Phone();
//			CLK_SysTickDelay(20000);
		}
}
