#include "lx_wall_clock_timer.h"
#include "lx_communicate_protocol.h"
#include "PowerDown.h"
#include "lx_lcd_show.h"

static UTCTime SecondCountRTC;      /*This value should be stored in flash*/

volatile S_RTC_TIME_DATA_T sCurTime;	
Lcd_Time_Show Blu_rev_time;
void RTC_IRQHandler(void)
{  			
	
//	  Leave_PowerDown();
    RTC_GetDateAndTime(&sCurTime); 
    Blu_rev_time.year=sCurTime.u32Year;
	  Blu_rev_time.month=sCurTime.u32Month;
	  Blu_rev_time.day=sCurTime.u32Day;
	  Blu_rev_time.hour=sCurTime.u32Hour;
	  Blu_rev_time.minutes=sCurTime.u32Minute;
	  Lcd_Set_Time(Blu_rev_time);
    RTC_CLEAR_TICK_INT_FLAG();	
}

void Init_RTC(void)
{
	  S_RTC_TIME_DATA_T sInitTime;
    // Time Setting
    sInitTime.u32Year       = 2016;
    sInitTime.u32Month      = 1;
    sInitTime.u32Day        = 24;
    sInitTime.u32Hour       = 15;
    sInitTime.u32Minute     = 20;
    sInitTime.u32Second     = 0;
    sInitTime.u32DayOfWeek  = RTC_TUESDAY;
    sInitTime.u32TimeScale  = RTC_CLOCK_24;
    RTC_Open(&sInitTime);
    RTC_SetTickPeriod(RTC_TICK_1_SEC);
    // Enable RTC Tick Interrupt
    RTC_EnableInt(RTC_RIER_TIER_Msk);
    NVIC_EnableIRQ(RTC_IRQn);	
	  // Enable RTC wakeup
		RTC_ENABLE_TICK_WAKEUP(); 
}

uint8_t monthLength( uint8_t lpyr, uint8_t mon )
{
    uint8_t days = 31;

    if ( mon == 1 ) // feb
    {
        days = ( 28 + lpyr );
    } else {
        if ( mon > 6 ) // aug-dec
        {
            mon--;
        }

        if ( mon & 1 ) {
            days = 30;
        }
    }

    return ( days );
}
 UTCTime get_wall_clock_time_counter(void)
{
               return SecondCountRTC;
}

UTCTime convert_time_to_Second(time_union_t time)
{
    uint32_t i = 0;
    UTCTime offset = 0;
    uint8_t leapYear;
    //day time
    offset += time.time.seconds;
    offset += time.time.minute * 60;
    offset += time.time.hours * 60 * 60;

    leapYear = IsLeapYear(time.time.year + 2000);

    offset += DAY * (time.time.day - 1);

    for(i = 0; i < time.time.month - 1; ++i) { //month start from 1
        offset += monthLength(leapYear,i) * DAY;
    }

    for(i = 0; i< time.time.year ;++i) {
        if(IsLeapYear(i + 2000)) {
            offset += DAY * 366;
        } else {
            offset += DAY * 365;
        }
    }

    return offset;

}

void set_system_clock(time_union_t time)
{
    uint32_t i = 0;
    UTCTime offset = 0;
    uint8_t leapYear;
    //day time
    offset += time.time.seconds;
    offset += time.time.minute * 60;
    offset += time.time.hours * 60 * 60;

    leapYear = IsLeapYear(time.time.year + 2000);

    offset += DAY * (time.time.day - 1);

    for(i = 0; i < time.time.month - 1; ++i) { //month start from 1
        offset += monthLength(leapYear,i) * DAY;
    }

    for(i = 0; i< time.time.year ;++i) {
        if(IsLeapYear(i + 2000)) {
            offset += DAY * 366;
        } else {
            offset += DAY * 365;
        }
    }

    SecondCountRTC = offset;

    /************************************************************
    * restart timer
    ************************************************************/
}

void system_clock_init(void)
{
    uint32_t err_code;

    /* set a default value */
    S_RTC_TIME_DATA_T sInitTime;
    // Time Setting
    sInitTime.u32Year       = 2016;
    sInitTime.u32Month      = 1;
    sInitTime.u32Day        = 24;
    sInitTime.u32Hour       = 15;
    sInitTime.u32Minute     = 20;
    sInitTime.u32Second     = 0;
    sInitTime.u32DayOfWeek  = RTC_TUESDAY;
    sInitTime.u32TimeScale  = RTC_CLOCK_24;

    /* Init ticks from RTC */
    SecondCountRTC = 0;       /* This should read from flash */
    //  00 42 00 40 01
    //  1000010000000000100000000000001
    RTC_Open(&sInitTime);
	  RTC_ENABLE_TICK_WAKEUP();
}